__title__ = 'slim.models.fields'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('LanguageField', 'SimpleLanguageField')

from six import PY2

from django.db import models
from django.core import exceptions
from django.utils.translation import ugettext_lazy as _

from slim import get_languages, default_language, get_languages_keys
from slim.settings import ENABLE_MONKEY_PATCHING
from slim.monkey_patches import monkeypatch_method, monkeypatch_property
from slim.helpers import admin_change_url, admin_add_url

class LanguageField(models.CharField):
    """
    LanguageField model. Stores language string in a ``CharField`` field.

    Using `contrib_to_class` melthod adds `translation_of` field, which is simply a ``ForeignKey``
    to the same class.
    """
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        """
        Create new field. Argument ``populate`` will be sent as-is to the form field.
        """
        defaults = {
            'verbose_name': _('Language'),
            'populate': None,
            'max_length': 10,
            'choices': get_languages(),
            'default': default_language
        }
        defaults.update(kwargs)
        self.populate = defaults.pop('populate', None)
        super(LanguageField, self).__init__(*args, **defaults)

    def formfield(self, **kwargs):
        """
        Returns best form field to represent this model field
        """
        defaults = {
            'form_class': LanguageField,
            'populate': self.populate,
        }
        defaults.update(kwargs)
        return super(models.CharField, self).formfield(**defaults)

    def validate(self, value, model_instance):
        """
        Validating the field.

        We shall make sure that there are double translations for the same language for the same object. That's
        why, in case if model is not yet saved (``translated_object`` does not yet have a primary key), we check
        if there are already translations of the same object in the language we specify now.

        Otherwise, if ``model_instance`` already has a primary key, we anyway try to get a ``translated_object``
        and compare it with our ``model_instance``. In case if ``translated_object`` exists and not equal to our
        ``model_instance`` we raise an error.

        NOTE: This has nothing to do with unique fields in the original ``model_instance``. Make sure you have
        properly specified all unique attributes with respect to ``LanguageField` of your original
        ``model_instance`` if you need those records to be unique.
        """
        if model_instance.original_translation:
            translated_object = model_instance.original_translation.get_translation_for(value)
        else:
            translated_object = None
        if not model_instance.pk:
            if translated_object and translated_object.pk:
                raise exceptions.ValidationError("Translation in language %s for this object already exists." % value)
        else:
            if translated_object and translated_object.pk and model_instance != translated_object:
                raise exceptions.ValidationError("Translation in language %s for this object already exists." % value)
        super(LanguageField, self).validate(value, model_instance)

    def contribute_to_class(self, cls, name):
        """
        Language field consists of more than one database record. We have ``lanaguage`` (CharField)
        and ``translation_of`` (ForeignKey to ``cls``) in order to identify translated and
        primary objects.

        We have a set of very useful methods implemented in order to get translations easily.
        """
        self.name = name
        self.translation_of = models.ForeignKey(cls, blank=True, null=True, verbose_name=_('Translation of'), \
                                                related_name='translations', \
                                                limit_choices_to={'language': default_language}, \
                                                help_text=_('Leave this empty for entries in the primary language.'))
        cls.add_to_class('translation_of', self.translation_of)
        super(LanguageField, self).contribute_to_class(cls, name)

        if ENABLE_MONKEY_PATCHING:
            @monkeypatch_property(cls)
            def is_multilingual(self):
                """
                Simple flat to use on objects to find our wheither they are multilinugal
                or not

                :return bool: Always returns boolean True
                """
                return True

            @monkeypatch_method(cls)
            def get_redirect_to_target(self, request):
                """
                Find an acceptable redirect target. If this is a local link, then try
                to find the page this redirect references and translate it according
                to the user's language. This way, one can easily implement a localized
                "/"-url to welcome page redirection.
                """
                target = self.redirect_to
                if target and target.find('//') == -1: # Not an offsite link http://bla/blubb
                    try:
                        page = cls.objects.page_for_path(target)
                        page = page.get_translation(getattr(request, 'LANGUAGE_CODE', None))
                        target = page.get_absolute_url()
                    except cls.DoesNotExist:
                        pass
                return target

            @monkeypatch_method(cls)
            def available_translations(self):
                """
                Returns available translations.

                :return interable: At this moment a list of objects.
                """
                if not self.id: # New, unsaved pages have no translations
                    return []
                if is_primary_language(self.language):
                    return self.translations.all()
                elif self.translation_of:
                    return [self.translation_of] + list(self.translation_of.translations.exclude(
                        language=self.language))
                else:
                    return []

            @monkeypatch_method(cls)
            def get_original_translation(self, *args, **kwargs):
                """
                Gets original translation of current object.

                :return obj: Object of the same class as the one queried.
                """
                if is_primary_language(self.language):
                    return self
                return self.translation_of

            @monkeypatch_method(cls)
            def translation_admin(self, *args, **kwargs):
                """
                Gets a HTML with URL to the original translation of available. For admin use.

                :return str:
                """
                if self.translation_of:
                    if PY2:
                        url_title = unicode(self.translation_of)
                    else:
                        url_title = self.translation_of

                    return admin_change_url(
                        self._meta.app_label,
                        self._meta.module_name,
                        self.translation_of.id,
                        url_title = url_title
                        )
                return ''
            translation_admin.allow_tags = True
            translation_admin.short_description = _('Translation of')

            @monkeypatch_method(cls)
            def _available_translations_admin(self, include_self=True):
                """
                Gets a HTML with all available translation URLs for current object if available. For admin use.

                :return str:
                """
                try:
                    original_translation = self.original_translation
                    available_translations = list(self.available_translations())
                    languages_keys = get_languages_keys()
                    languages = dict(get_languages())

                    if include_self:
                        available_translations.append(self)

                    output = []
                    # Processing all available translations. Adding edit links.
                    if available_translations:
                        for translation in available_translations:
                            if PY2:
                                url_title = unicode(languages[translation.language])
                            else:
                                url_title = languages[translation.language]
                            output.append(
                                admin_change_url(
                                    translation._meta.app_label,
                                    translation._meta.module_name,
                                    translation.id,
                                    url_title = url_title
                                    )
                                )
                            languages_keys.remove(translation.language)

                    if self.pk and self.language in languages_keys:
                        languages_keys.remove(self.language)

                    # For all languages that are still available (original object has no translations for)
                    for language in languages_keys:
                        url = admin_add_url(
                                self._meta.app_label,
                                self._meta.module_name,
                                '?translation_of=%s&amp;language=%s' % (str(original_translation.id), language)
                                )
                        if PY2:
                            name = unicode(languages[language])
                        else:
                            name = languages[language]

                        output.append(u'<a href="%(url)s" style="color:#baa">%(name)s</a>' % {'url': url, 'name': name})
                    return ' | '.join(output)
                except Exception as e:
                    return ''

            @monkeypatch_method(cls)
            def available_translations_admin(self, *args, **kwargs):
                """
                Gets a HTML with all available translation URLs for current object if available. For admin use.

                :return str:
                """
                return self._available_translations_admin(include_self=True, *args, **kwargs)
            available_translations_admin.allow_tags = True
            available_translations_admin.short_description = _('Translations')

            @monkeypatch_method(cls)
            def available_translations_exclude_current_admin(self, *args, **kwargs):
                """
                Same as `available_translations_admin` but does not include itself to the list.

                :return str:
                """
                return self._available_translations_admin(include_self=False, *args, **kwargs)
            available_translations_exclude_current_admin.allow_tags = True
            available_translations_exclude_current_admin.short_description = _('Translations')

            @monkeypatch_property(cls)
            def original_translation(self):
                """
                Property for ``get_original_translation`` method.

                :return obj: Object of the same class as the one queried.
                """
                return self.get_original_translation()

            @monkeypatch_method(cls)
            def get_translation_for(self, language):
                """
                Get translation article in given language.

                :param str language: Which shall be one of the languages specified in ``LANGUAGES``
                    in `settings.py`.
                :return obj: Either object of the same class as or None if no translations are
                    available for the given ``language``.
                """
                if not language in get_languages_keys():
                    return None
                if str(self.language) == str(language):
                    return self
                if str(self.original_translation.language) == str(language):
                    return self.original_translation
                try:
                    return self.original_translation.translations.get(language=language)
                except Exception as e:
                    return None


class SimpleLanguageField(models.CharField):
    """
    SimpleLanguageField model. Stores language string in a ``CharField`` field.
    """

    def __init__(self, *args, **kwargs):
        """
        Create new field. Argument ``populate`` will be sent as-is to the form field.
        """
        defaults = {
            'verbose_name': _('Language'),
            'populate': None,
            'max_length': 10,
            'choices': get_languages(),
            'default': default_language
        }
        defaults.update(kwargs)
        self.populate = defaults.pop('populate', None)
        super(SimpleLanguageField, self).__init__(*args, **defaults)

    def formfield(self, **kwargs):
        """
        Returns best form field to represent this model field
        """
        defaults = {
            'form_class': SimpleLanguageField,
            'populate': self.populate,
        }
        defaults.update(kwargs)
        return super(models.CharField, self).formfield(**defaults)

# Add schema's for South
try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules(
        rules=[((LanguageField,), [], {'populate': ('populate', {'default':None}),})], \
        patterns=['slim.models\.fields\.LanguageField', 'slim.models\.fields\.SimpleLanguageField']
        )
except ImportError:
    pass



__title__ = 'slim.utils'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('locale_url_is_installed',)

from django.conf import settings

from slim.settings import USE_LOCALEURL


def locale_url_is_installed():
    """
    Checks if localeurl is installed in the Django project.

    :return bool:
    """
    if USE_LOCALEURL is True and 'localeurl' in settings.INSTALLED_APPS and \
       'localeurl.middleware.LocaleURLMiddleware' in settings.MIDDLEWARE_CLASSES:
       return True
    return False



__title__ = 'slim.helpers'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('get_default_language', 'default_language', 'get_languages', 'get_languages_keys', \
           'get_language_from_request', 'get_languages_dict', 'admin_change_url', 'admin_add_url', 'smart_resolve')

from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.translation import get_language_info

from slim.settings import USE_LOCAL_LANGUAGE_NAMES

def get_default_language():
    """
    Gets default language.

    :return str:
    """
    return settings.LANGUAGES[0][0]
default_language = get_default_language()


def get_languages():
    """
    Gets available languages.

    :return iterable:
    """
    if not USE_LOCAL_LANGUAGE_NAMES:
        return settings.LANGUAGES
    else:
        languages = []
        for lang_code, lang_name in settings.LANGUAGES:
            try:
                lang_name = get_language_info(lang_code)['name_local']
            except Exception as e:
                pass
            languages.append((lang_code, lang_name))
        return languages


def get_languages_keys():
    """
    Returns just languages keys.

    :return list:
    """
    return [key for key, name in get_languages()]


def get_languages_dict():
    """
    Returns just languages dict.

    :return dict:
    """
    return dict(get_languages())


def get_language_from_request(request, default=default_language):
    """
    Gets language from HttpRequest

    :param django.http.HttpRequest:
    :param str default:
    :return str:
    """
    if hasattr(request, 'LANGUAGE_CODE') and request.LANGUAGE_CODE:
        return request.LANGUAGE_CODE
    else:
        return default


def admin_change_url(app_label, module_name, object_id, extra_path='', url_title=None):
    """
    Gets an admin change URL for the object given.

    :param str app_label:
    :param str module_name:
    :param int object_id:
    :param str extra_path:
    :param str url_title: If given, an HTML a tag is returned with `url_title` as the tag title. If left to None
        just the URL string is returned.
    :return str:
    """
    try:
        url = reverse('admin:%s_%s_change' %(app_label, module_name), args=[object_id]) + extra_path
        if url_title:
            return u'<a href="%s">%s</a>' %(url, url_title)
        else:
            return url
    except:
        return None


def admin_add_url(app_label, module_name, extra_path='', url_title=None):
    """
    Gets an admin edit URL for the object given.

    :param str app_label:
    :param str module_name:
    :param str extra_path:
    :param str url_title: If given, an HTML a tag is returned with `url_title` as the tag title. If left to None
        just the URL string is returned.
    :return str:
    """
    try:
        url = reverse('admin:%s_%s_add' %(app_label, module_name)) + extra_path
        if url_title:
            return u'<a href="%s">%s</a>' %(url, url_title)
        else:
            return url
    except:
        return None


def smart_resolve(var, context):
    """
    Resolves variable from context in a smart way. First trying to resolve from context
    and when result is None checks if variable is not None and returns just variable
    when not. Otherwise returns None.

    :param str var:
    :param Context context:
    :return mixed:
    """
    if var is None:
        return None

    ret_val = None
    try:
        ret_val = var.resolve(context, True)
    except:
        ret_val = var
    if ret_val is None:
        ret_val = var

    return ret_val



__title__ = 'slim.admin'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SlimAdmin',)

from django.utils.translation import ugettext_lazy as _
from django.contrib import admin

from slim.helpers import default_language


class SlimAdmin(admin.ModelAdmin):
    """
    SlimAdmin.

    ``list_view_primary_only`` - if set to True, onlt primary language items would be shown in the list
    view. Default value is False.

    ``language_field`` - name of the language field defined in your model. Default value `language`.

    ``auto_add_edit_view`` - if set to True, extra fields for language editing are added to the list view.
    Do NOT set this value to False!

    ``collapse_slim_fieldset`` if set to True, the language fieldset is shown collapsed.
    """
    # If set to True, only primary language objects are shown in the list view.
    list_view_primary_only = False

    # Language field in your model
    language_field = 'language'

    # If set to True, languages fields are added to the edit view (as a fieldset)
    auto_add_edit_view = True

    # If set to True, languages fieldset is auto added to the list view
    auto_add_list_view = True

    # If set to True, the fieldset is shown collapsed.
    collapse_slim_fieldset = True

    def queryset(self, *args, **kwargs):
        # For faster admin load we use ``prefetch_related``. Note, that this doesn't work on Django < 1.5.
        queryset = super(SlimAdmin, self).queryset(*args, **kwargs) \
                                         .prefetch_related('translations') \
                                         .select_related('translation_of')

        if self.list_view_primary_only is True:
            f = {self.language_field: default_language}
            queryset = queryset.filter(**f)

        return queryset

    def get_list_display(self, *args, **kwargs):
        list_display = super(SlimAdmin, self).get_list_display(*args, **kwargs)

        if self.auto_add_list_view:
            list_display = list(list_display)
            list_display.extend(
                (self.language_field, 'available_translations_admin')
            )

        return list_display

    def get_readonly_fields(self, *args, **kwargs):
        readonly_fields = super(SlimAdmin, self).get_readonly_fields(*args, **kwargs)

        if self.auto_add_list_view:
            readonly_fields = list(readonly_fields)
            readonly_fields.extend(
                ('available_translations_exclude_current_admin',)
            )

        return readonly_fields

    def get_list_filter(self, *args, **kwargs):
        list_filter = super(SlimAdmin, self).get_list_filter(*args, **kwargs)

        if list_filter is None:
            return [self.language_field]
        else:
            list_filter = list(list_filter)
            list_filter.append(self.language_field)

        return list_filter

    def _declared_fieldsets(self, *args, **kwargs):
        declared_fieldsets = super(SlimAdmin, self)._declared_fieldsets(*args, **kwargs)

        if self.auto_add_edit_view:
            declared_fieldsets = list(declared_fieldsets)
            declared_fieldsets.append(
                (_("Translations"), {
                    'classes': ('collapse' if self.collapse_slim_fieldset else '',),
                    'fields': (self.language_field, 'translation_of', 'available_translations_exclude_current_admin',)
                }),
            )

        return declared_fieldsets
    declared_fieldsets = property(_declared_fieldsets)



__title__ = 'slim.exceptions'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('LocaleurlImportError',)

class LocaleurlImportError(ImportError):
    pass



__title__ = 'slim.models.__init__'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('Slim', 'SlimBaseModel')

from six import PY2

from django.db import models
from django.utils.translation import ugettext_lazy as _

from slim import get_languages, default_language, get_languages_keys
from slim.translations import is_primary_language
from slim.helpers import admin_change_url, admin_add_url

class Slim(object):
    """
    Add this class to all your multi-lingual Django models, where you use ``slim.models.fields.LanguageField``.
    Alternatively, you may use the ``slim.models.SlimBaseModel``.
    """
    @property
    def is_multilingual(self):
        """
        Simple flat to use on objects to find our wheither they are multilinugal
        or not

        :return bool: Always returns boolean True
        """
        return True

    def get_redirect_to_target(self, request):
        """
        Find an acceptable redirect target. If this is a local link, then try
        to find the page this redirect references and translate it according
        to the user's language. This way, one can easily implement a localized
        "/"-url to welcome page redirection.
        """
        target = self.redirect_to
        if target and target.find('//') == -1: # Not an offsite link http://bla/blubb
            try:
                page = cls.objects.page_for_path(target)
                page = page.get_translation(getattr(request, 'LANGUAGE_CODE', None))
                target = page.get_absolute_url()
            except cls.DoesNotExist:
                pass
        return target

    def available_translations(self):
        """
        Returns available translations.

        :return interable: At this moment a list of objects.
        """
        if not self.id: # New, unsaved pages have no translations
            return []
        if is_primary_language(self.language):
            return self.translations.all()
        elif self.translation_of:
            return [self.translation_of] + list(self.translation_of.translations.exclude(
                language=self.language))
        else:
            return []

    def get_original_translation(self, *args, **kwargs):
        """
        Gets original translation of current object.

        :return obj: Object of the same class as the one queried.
        """
        if is_primary_language(self.language):
            return self
        return self.translation_of

    def translation_admin(self, *args, **kwargs):
        """
        Gets a HTML with URL to the original translation of available. For admin use.

        :return str:
        """
        if self.translation_of:
            if PY2:
                url_title = unicode(self.translation_of)
            else:
                url_title = self.translation_of

            return admin_change_url(
                self._meta.app_label,
                self._meta.module_name,
                self.translation_of.id,
                url_title = url_title
                )
        return ''
    translation_admin.allow_tags = True
    translation_admin.short_description = _('Translation of')

    def _available_translations_admin(self, include_self=True):
        """
        Gets a HTML with all available translation URLs for current object if available. For admin use.

        :return str:
        """
        try:
            original_translation = self.original_translation
            available_translations = list(self.available_translations())
            languages_keys = get_languages_keys()
            languages = dict(get_languages())

            if include_self:
                available_translations.append(self)

            output = []
            # Processing all available translations. Adding edit links.
            if available_translations:
                for translation in available_translations:
                    if PY2:
                        url_title = unicode(languages[translation.language])
                    else:
                        url_title = languages[translation.language]

                    output.append(
                        admin_change_url(
                            translation._meta.app_label,
                            translation._meta.module_name,
                            translation.id,
                            url_title = url_title
                            )
                        )
                    languages_keys.remove(translation.language)

            if self.pk and self.language in languages_keys:
                languages_keys.remove(self.language)

            # For all languages that are still available (original object has no translations for)
            for language in languages_keys:
                url = admin_add_url(
                        self._meta.app_label,
                        self._meta.module_name,
                        '?translation_of=%s&amp;language=%s' % (str(original_translation.id), language)
                        )
                if PY2:
                    name = unicode(languages[language])
                else:
                    name = languages[language]

                output.append(u'<a href="%(url)s" style="color:#baa">%(name)s</a>' % {'url': url, 'name': name})
            return u' | '.join(output)
        except Exception as e:
            return u''

    def available_translations_admin(self, *args, **kwargs):
        """
        Gets a HTML with all available translation URLs for current object if available. For admin use.

        :return str:
        """
        return self._available_translations_admin(include_self=True, *args, **kwargs)
    available_translations_admin.allow_tags = True
    available_translations_admin.short_description = _('Translations')

    def available_translations_exclude_current_admin(self, *args, **kwargs):
        """
        Same as `available_translations_admin` but does not include itself to the list.

        :return str:
        """
        return self._available_translations_admin(include_self=False, *args, **kwargs)
    available_translations_exclude_current_admin.allow_tags = True
    available_translations_exclude_current_admin.short_description = _('Translations')

    @property
    def original_translation(self):
        """
        Property for ``get_original_translation`` method.

        :return obj: Object of the same class as the one queried.
        """
        return self.get_original_translation()

    def get_translation_for(self, language):
        """
        Get translation article in given language.

        :param str language: Which shall be one of the languages specified in ``LANGUAGES``
            in `settings.py`.
        :return obj: Either object of the same class as or None if no translations are
            available for the given ``language``.
        """
        if not language in get_languages_keys():
            return None
        if str(self.language) == str(language):
            return self
        if str(self.original_translation.language) == str(language):
            return self.original_translation
        try:
            return self.original_translation.translations.get(language=language)
        except Exception as e:
            return None

class SlimBaseModel(models.Model, Slim):
    """
    An abstract Django model.
    """
    class Meta:
        abstract = True



__title__ = 'slim.models.decorators'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('prepend_language', 'localeurl_prepend_language', 'auto_prepend_language')

from slim.utils import locale_url_is_installed

def prepend_language(func, language_field='language'):
    """
    Prepends the language from the model to the path resolved.
    """
    def inner(self, *args, **kwargs):
        return "/%s%s" % (getattr(self, language_field), func(self, *args, **kwargs))
    return inner

try:
    from localeurl.templatetags.localeurl_tags import chlocale

    def localeurl_prepend_language(func, language_field='language'):
        """
        Prepends the language from the model to the path resolved when `django-localeurl` package is used.
        """
        def inner(self, *args, **kwargs):
            return chlocale(func(self, *args, **kwargs), getattr(self, language_field))
        return inner

    # If `localeurl` is available gets the `localeurl` based decorator. Otherwise, gets `prepend_language`
    # based decorator.
    if locale_url_is_installed():
        auto_prepend_language = localeurl_prepend_language
    else:
        auto_prepend_language = prepend_language

except ImportError as e:
    from slim.exceptions import LocaleurlImportError

    def localeurl_prepend_language(func, language_field=None):
        raise LocaleurlImportError(
            "You should have localeurl installed in order to use "
            "`slim.models.localeurl_prepend_language` decorator."
            )
    # Fallback
    auto_prepend_language = prepend_language



__title__ = 'slim'
__version__ = '0.7.3'
__build__ = 0x000012
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'

from slim.helpers import *
from slim.models import *
from slim.models.fields import *



__title__ = 'slim.settings'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('USE_LOCALEURL', 'USE_LOCAL_LANGUAGE_NAMES', 'ENABLE_MONKEY_PATCHING')

from slim.conf import get_setting

USE_LOCALEURL = get_setting('USE_LOCALEURL')
USE_LOCAL_LANGUAGE_NAMES = get_setting('USE_LOCAL_LANGUAGE_NAMES')
ENABLE_MONKEY_PATCHING = get_setting('ENABLE_MONKEY_PATCHING')



__title__ = 'slim.translations'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('short_language_code', 'is_primary_language')

from django.utils import translation

from slim.helpers import default_language


def short_language_code(code=None):
    """
    Extracts the short language code from its argument (or return the default language code).

    :param str code:
    :return str:

    from django.conf import settings
    >>> short_language_code('de')
    'de'
    >>> short_language_code('de-at')
    'de'
    """
    if code is None:
        code = translation.get_language()

    pos = code.find('-')
    if pos > -1:
        return code[:pos]
    return code


def is_primary_language(language=None):
    """
    Returns true if current or passed language is the primary language for this site.
    (The primary language is defined as the first language in settings.LANGUAGES.)

    :param str language:
    :return bool:
    """
    if not language:
        language = translation.get_language()

    return language == default_language



__title__ = 'slim.templatetags.slim_tags'
__version__ = '0.7'
__build__ = 0x000007
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__all__ = ('get_translated_object_for', 'get_translated_objects_for', 'set_language', 'multiling_is_enabled',
           'slim_language_name')

import re

from django.utils.translation import ugettext_lazy as _
from django import template
from django.template import Library
from django.utils import translation

from slim.helpers import smart_resolve, default_language, get_language_from_request, get_languages_keys
from slim.helpers import get_languages_dict

register = Library()


class GetTranslatedObjectForNode(template.Node):
    """
    Node for template tag ``get_translated_object_for``.
    """
    def __init__(self, obj, as_var, language=None):
        """
        :param obj: Object to translate.
        :param string as_var: Desired variable name in the template context.
        :param str language: Language code. Must be one of those defined in ``LANGUAGES``
            of "settigns.py"
        :raise template.TemplateSyntaxError: If invalid syntax or in case if HttpRequest
            can't be retrieved from template context, while ``language`` attribute is not
            specified.
        """
        self.as_var = as_var
        self.obj = obj
        self.language = language

    def render(self, context):
        ret_val = None
        obj = smart_resolve(self.obj, context)

        try:
            is_multilingual = obj.is_multilingual
        except AttributeError as e:
            is_multilingual = False
            raise template.TemplateSyntaxError(
                _("Invalid usage of ``get_translated_object_for``. Translated object shall be multilingual.")
                )

        try:
            request = context['request']
        except:
            request = None

        if language is None and request is None:
            raise template.TemplateSyntaxError(
                _("Invalid usage of ``get_translated_object_for``. Can't retrieve ``HttpRequest`` object from "
                   "template context, while ``language`` attribute is not specified.")
                )

        if language is None:
            language = get_language_from_request(request)
        else:
            language = smart_resolve(self.language, context)
            if language:
                language = str(language)
            else:
                language = default_language

        ret_val = obj.get_translation_for(language)

        if self.as_var:
            context[self.as_var] = ret_val
            return ''
        return ret_val


@register.tag
def get_translated_object_for(parser, token):
    """
    Gets translated object for the object given.

    Syntax::
        {% get_translated_object_for [object] language=[language] as [var_name] %}

    Example usage::
        {% get_translated_object_for article as translated_article %}
        {% get_translated_object_for article language=ru as translated_article %}
    """
    bits = token.contents.split()
    if 'as' != bits[-2]:
        raise template.TemplateSyntaxError(
            _("Invalid syntax for %s. You must specify a name for translated object." % bits[0])
            )
    as_var = bits[-1]
    try:
        obj = parser.compile_filter(bits[1])
    except:
        template.TemplateSyntaxError(_("Invalid syntax for %s. You must provide an object to translate." % bits[0]))

    m = re.search(r'language=(\w+)', ' '.join(bits[2:-2]))

    if m:
        language = parser.compile_filter(m.groups()[0])
    else:
        language = None

    return GetTranslatedObjectForNode(obj=obj, as_var=as_var, language=language)


class GetTranslatedObjectsForNode(template.Node):
    """
    Node for template tag ``get_translated_objects_for``.
    """
    def __init__(self, obj, as_var):
        """
        :param obj: Object to get translations for.
        :param str as_var: Desired variable name in the template context.
        :raise template.TemplateSyntaxError: If invalid syntax or in case if HttpRequest
            can't be retrieved from template context, while ``language`` attribute is not
            specified.
        """
        self.as_var = as_var
        self.obj = obj

    def render(self, context):
        ret_val = None
        obj = smart_resolve(self.obj, context)

        try:
            is_multilingual = obj.is_multilingual
        except AttributeError as e:
            is_multilingual = False
            raise template.TemplateSyntaxError(
                _("Invalid usage of get_translated_object_for. Translated object shall be multilingual.")
                )

        ret_val = obj.available_translations()

        if self.as_var:
            context[self.as_var] = ret_val
            return ''
        return ret_val


@register.tag
def get_translated_objects_for(parser, token):
    """
    Gets translations available for the given object.

    Syntax::
        {% get_translated_objects_for [object] as [var_name] %}

    Example usage::
        {% get_translated_objects_for article as translated_article %}
    """
    bits = token.contents.split()
    if 'as' != bits[-2]:
        raise template.TemplateSyntaxError(
            _("Invalid syntax for %s. You must specify a name for translated object." % bits[0])
            )
    as_var = bits[-1]
    try:
        obj = parser.compile_filter(bits[1])
    except:
        template.TemplateSyntaxError(_("Invalid syntax for %s. You must provide an object to translate." % bits[0]))

    return GetTranslatedObjectsForNode(obj=obj, as_var=as_var)


class SetLanguageNode(template.Node):
    """
    Node for ``set_language`` tag.
    """
    def __init__(self, language=None):
        self.language = language

    def render(self, context):
        # Try to get request.LANGUAGE_CODE. If fail, use default one.
        request = context['request']
        language = get_language_from_request(request, default=None)

        if not language:
            language = smart_resolve(self.language, context)
            if not language in get_languages_keys():
                language = default_language

        translation.activate(language)
        return ''


@register.tag
def set_language(parser, token):
    """
    Sets current language code.

    FIXME: This is actually a hack.

    Syntax::
        {% set_language [language] %}
    Example::
        {% set_language ru %}
    """
    bits = token.contents.split()
    if 2 < len(bits):
        raise template.TemplateSyntaxError("'%s' tag takes one argument at most" % bits[0])
    elif 2 == len(bits):
        language = parser.compile_filter(bits[1])
    else:
        language = None
    return SetLanguageNode(language=language)


class MultilinIsEnabledNode(template.Node):
    """
    Node for ``multiling_is_enabled`` tag.
    """
    def __init__(self, as_var=None):
        self.as_var = as_var

    def render(self, context):
        # Try to get request.LANGUAGE_CODE. If fail, use default one.
        if len(get_languages_keys()) > 1:
            ret_val = True
        else:
            ret_val = False

        if self.as_var:
            context[self.as_var] = ret_val
            return ''
        return ret_val


@register.tag
def multiling_is_enabled(parser, token):
    """
    Checks if multiling shall be enabled (in templates). Simply, if LANGUAGES tuple contains more than one
    language, we return boolean True; otherwise - boolean False.

    Syntax::
        {% multiling_is_enabled as [var_name] %}
    Example::
        {% multiling_is_enabled as multiling_is_enabled %}
    """
    bits = token.contents.split()
    if not len(bits) in (1, 3):
        raise template.TemplateSyntaxError("'%s' tag takes five arguments at most" % bits[0])

    if 3 == len(bits):
        if 'as' != bits[-2]:
            raise template.TemplateSyntaxError(
                _("Invalid syntax for %s. You must specify a name for translated object." % bits[0])
                )
        as_var = bits[-1]
    else:
        as_var = None

    return MultilinIsEnabledNode(as_var=as_var)


@register.filter
def slim_language_name(lang_code):
    """
    Not all languages are available in Django yet. It might happen that support for your own precious language is
    not yet available in Django and many apps rely on languages list defined in `django.conf.global_settings` module.

    Thus, to have translations for your own language available, the following approach is introduced:
    - Pick the language code closest to your language but name it differently: (('ar', ugettext('Armenian')),).
    - Instead of using Django's `language_name` filter (of `i18n` module) use `slim_language_name` filter just the
      same way.

    This filter would get your item tranlation based on your project translations.

    :param str lang_code:
    :return str:
    """
    return _(get_languages_dict()[lang_code])


@register.filter
def slim_language_local_name(lang_code):
    """
    Localised language name.

    :param str lang_code:
    :return str:
    """
    try:
        return translation.get_language_info(lang_code)['name_local']
    except:
        return lang_code




__title__ = 'slim.defaults'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('USE_LOCALEURL', 'USE_LOCAL_LANGUAGE_NAMES', 'ENABLE_MONKEY_PATCHING')

# If set to False, `django-localeurl` usage in `slim` is force-disabled.
USE_LOCALEURL = True

# If set to True, local language names are used.
USE_LOCAL_LANGUAGE_NAMES = False

# If set to True, class methods (``snart.models.Snart`` are monkey patched to the field, thus you don't have to
# inherit from snart models.
ENABLE_MONKEY_PATCHING = False


import sys
import os
from setuptools import setup, find_packages

try:
  readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
except:
  readme = ''

version = '0.7.3'

install_requires = [
    'six>=1.4.1,<=1.6.1',
]

try:
    PY2 = sys.version_info[0] == 2
    PY3 = sys.version_info[0] == 3
    if PY2:
        install_requires.append('django-localeurl==2.0.1')
except:
    pass

setup(
    name = 'django-slim',
    version = version,
    description = ("Simple implementation of multi-lingual models for Django."),
    long_description = readme,
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)",
    ],
    keywords = 'multi-lingual, django, python',
    author = 'Artur Barseghyan',
    author_email = 'artur.barseghyan@gmail.com',
    package_dir = {'':'src'},
    packages = find_packages(where='./src'),
    url = 'https://github.com/barseghyanartur/django-slim',
    license = 'GPL 2.0/LGPL 2.1',
    install_requires = install_requires
)



from __future__ import print_function

__title__ = 'slim.tests'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'

import unittest
import os

from six import text_type

PROJECT_DIR = lambda base : os.path.join(os.path.dirname(__file__), base).replace('\\','/')

PRINT_INFO = True

def print_info(func):
    """
    Prints some useful info.
    """
    if not PRINT_INFO:
        return func

    def inner(self, *args, **kwargs):
        result = func(self, *args, **kwargs)

        print('\n\n{0}'.format(func.__name__))
        print('============================')
        if func.__doc__:
            print('""" {0} """'.format(func.__doc__.strip()))
        print('----------------------------')
        if result is not None:
            print(result)
        print('\n++++++++++++++++++++++++++++')

        return result
    return inner

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

import sys

PY3 = sys.version_info[0] == 3

_ = lambda s: s

# Skipping from non-Django tests.
if os.environ.get("DJANGO_SETTINGS_MODULE", None):

    from foo.models import FooItem

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    class SlimTest(unittest.TestCase): #unittest.TestCase
        """
        Tests of ``slim.models`` module.
        """

        def setUp(self):
            super(SlimTest, self).setUp()

            self.FOO_ITEM_EN_TITLE = "Foo title EN"
            self.FOO_ITEM_EN_BODY = "Foo body EN"
            self.FOO_ITEM_EN_SLUG = "foo-title-en"
            self.FOO_ITEM_EN_LANGUAGE = "en"

            self.FOO_ITEM_HY_TITLE = "Foo title HY"
            self.FOO_ITEM_HY_BODY = "Foo body HY"
            self.FOO_ITEM_HY_SLUG = "foo-title-hy"
            self.FOO_ITEM_HY_LANGUAGE = "hy"

            self.FOO_ITEM_NL_TITLE = "Foo title NL"
            self.FOO_ITEM_NL_BODY = "Foo body NL"
            self.FOO_ITEM_NL_SLUG = "foo-title-nl"
            self.FOO_ITEM_NL_LANGUAGE = "nl"

            self.FOO_ITEM_RU_TITLE = "Foo title RU"
            self.FOO_ITEM_RU_BODY = "Foo body RU"
            self.FOO_ITEM_RU_SLUG = "foo-title-ru"
            self.FOO_ITEM_RU_LANGUAGE = "ru"

        def __get_or_create_foo_items(self):
            # *************************************
            # ******** Creating main item *********
            # *************************************
            try:
                foo_item_en = FooItem._default_manager.get(slug=self.FOO_ITEM_EN_SLUG)

            except Exception as e:
                foo_item_en = FooItem(
                    title = self.FOO_ITEM_EN_TITLE,
                    body = self.FOO_ITEM_EN_BODY,
                    slug = self.FOO_ITEM_EN_SLUG,
                    language = self.FOO_ITEM_EN_LANGUAGE
                    )
                foo_item_en.save()

            # *************************************
            # ********* Armenian translation ******
            # *************************************
            try:
                foo_item_hy = FooItem._default_manager.get(slug=self.FOO_ITEM_HY_SLUG)

            except Exception as e:
                foo_item_hy = FooItem(
                    title = self.FOO_ITEM_HY_TITLE,
                    body = self.FOO_ITEM_HY_BODY,
                    slug = self.FOO_ITEM_HY_SLUG,
                    language = self.FOO_ITEM_HY_LANGUAGE,
                    translation_of = foo_item_en
                    )
                foo_item_hy.save()

                # *************************************
                # ********* Dutch translation *********
                # *************************************
            try:
                foo_item_nl = FooItem._default_manager.get(slug=self.FOO_ITEM_NL_SLUG)

            except Exception as e:
                foo_item_nl = FooItem(
                    title = self.FOO_ITEM_NL_TITLE,
                    body = self.FOO_ITEM_NL_BODY,
                    slug = self.FOO_ITEM_NL_SLUG,
                    language = self.FOO_ITEM_NL_LANGUAGE,
                    translation_of = foo_item_en
                    )
                foo_item_nl.save()

                # *************************************
                # ********* Russian translation *******
                # *************************************
            try:
                foo_item_ru = FooItem._default_manager.get(slug=self.FOO_ITEM_RU_SLUG)

            except Exception as e:
                foo_item_ru = FooItem(
                    title = self.FOO_ITEM_RU_TITLE,
                    body = self.FOO_ITEM_RU_BODY,
                    slug = self.FOO_ITEM_RU_SLUG,
                    language = self.FOO_ITEM_RU_LANGUAGE,
                    translation_of = foo_item_en
                    )
                foo_item_ru.save()

            return (foo_item_en, foo_item_hy, foo_item_nl, foo_item_ru)

        @print_info
        def test_01_get_translations(self):
            """
            Test ``get_translation_for`` method.
            """
            flow = []

            foo_item_en, foo_item_hy, foo_item_nl, foo_item_ru = self.__get_or_create_foo_items()

            flow.append(foo_item_en)

            # Test Armenian translation
            self.assertTrue(foo_item_en.get_translation_for('hy') == foo_item_hy)

            flow.append(foo_item_hy)

            # Test Dutch translation
            self.assertTrue(foo_item_en.get_translation_for('nl') == foo_item_nl)

            flow.append(foo_item_nl)

            # Test Russian translation
            self.assertTrue(foo_item_en.get_translation_for('ru') == foo_item_ru)

            flow.append(foo_item_ru)

            return flow

        @print_info
        def test_02_available_translations(self):
            """
            Test ``available_translations`` method.
            """
            flow = []

            foo_item_en, foo_item_hy, foo_item_nl, foo_item_ru = self.__get_or_create_foo_items()

            available_translations = [foo_item.pk for foo_item in foo_item_en.available_translations()]
            available_translations.sort()

            translations = [foo_item_hy.pk, foo_item_nl.pk, foo_item_ru.pk]
            translations.sort()

            self.assertTrue(available_translations == translations)

            flow.append(available_translations)
            flow.append(translations)

            return flow

        @print_info
        def test_03_auto_prepend_language_model_decorator(self):
            """
            Test the ``auto_prepend_language`` model decorator.
            """
            flow = []

            foo_item_en, foo_item_hy, foo_item_nl, foo_item_ru = self.__get_or_create_foo_items()

            self.assertTrue(
                text_type(foo_item_en.get_absolute_url()) == \
                text_type('/{0}/foo/{1}/').format(self.FOO_ITEM_EN_LANGUAGE, self.FOO_ITEM_EN_SLUG)
                )

            flow.append(foo_item_en.get_absolute_url())

            self.assertTrue(
                text_type(foo_item_hy.get_absolute_url()) == \
                text_type('/{0}/foo/{1}/').format(self.FOO_ITEM_HY_LANGUAGE, self.FOO_ITEM_HY_SLUG)
                )

            flow.append(foo_item_hy.get_absolute_url())

            self.assertTrue(
                text_type(foo_item_nl.get_absolute_url()) == \
                text_type('/{0}/foo/{1}/').format(self.FOO_ITEM_NL_LANGUAGE, self.FOO_ITEM_NL_SLUG)
                )

            flow.append(foo_item_nl.get_absolute_url())

            self.assertTrue(
                text_type(foo_item_ru.get_absolute_url()) == \
                text_type('/{0}/foo/{1}/').format(self.FOO_ITEM_RU_LANGUAGE, self.FOO_ITEM_RU_SLUG)
                )

            flow.append(foo_item_ru.get_absolute_url())

            return flow

        @print_info
        def test_04_original_translation(self):
            """
            Test ``original_translation`` method.
            """
            flow = []

            foo_item_en, foo_item_hy, foo_item_nl, foo_item_ru = self.__get_or_create_foo_items()

            self.assertTrue(foo_item_hy.original_translation == foo_item_en)
            self.assertTrue(foo_item_nl.original_translation == foo_item_en)
            self.assertTrue(foo_item_ru.original_translation == foo_item_en)

            #return flow


if __name__ == "__main__":
    # Tests
    unittest.main()